// The xMule Project - A Peer-2-Peer File Sharing Program
//
// Copyright (C) 2003-2006 Theodore R. Smith ( hopeseekr@gmail.com / http://www.xmule.ws/ )
// Copyright (C) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of Version 2 of the GNU General Public
// License as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

#ifdef PRECOMP
    #include "xmule-headers.h"
#endif

#include "SharedFilesWnd.h"       // Interface Declarations
#include "KnownFileList.h"        // CKnownFileList
#include "muuli_wdr.h"
#include "otherfunctions.h"
#include "SharedFileList.h"       // CSharedFileList
#include "wintypes.h"
#include "xmule.h"


#include <wx/settings.h>
#include <wx/sizer.h>             // wxSizer
#include <wx/stattext.h>          // wxStaticText
#include <sstream>

#define GetDlgItem(X) (wxStaticCast(wxWindow::FindWindowById((X)),wxStaticText))

BEGIN_EVENT_TABLE(CSharedFilesWnd, wxPanel)
    EVT_ODLIST_ITEM_SELECTED(ID_SHFILELIST, CSharedFilesWnd::OnLvnItemActivateSflist)
    EVT_BUTTON(IDC_RELOADSHAREDFILES, CSharedFilesWnd::OnBnClickedReloadsharedfiles)
END_EVENT_TABLE()

CSharedFilesWnd::CSharedFilesWnd(wxWindow *pParent) : wxPanel(pParent, wxID_ANY)
{
    memset(shownFileHash, 0, sizeof shownFileHash);
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWFRAME));
    wxSizer *content = sharedfilesDlg(this, true);
    content->Show(this, true);
    pop_bar = (wxGauge *) FindWindowByName(wxT("popbar"));
    pop_baraccept = (wxGauge *) FindWindowByName(wxT("popbarAccept"));
    pop_bartrans = (wxGauge *) FindWindowByName(wxT("popbarTrans"));
    sharedfilesctrl = (CSharedFilesCtrl *)(FindWindowByName(wxT("sharedFilesCt")));
}

CSharedFilesWnd::~CSharedFilesWnd()
{
}

// CSharedFilesWnd message handlers

void CSharedFilesWnd::OnBnClickedReloadsharedfiles(wxCommandEvent& event)
{
    theApp.sharedfiles->Reload(true, false);
}

void CSharedFilesWnd::Check4StatUpdate(CKnownFile *file)
{
    if (!memcmp(file->GetFileHash(), shownFileHash, 16)) ShowDetails(file);
}

void CSharedFilesWnd::OnLvnItemActivateSflist(wxODListEvent &evt)
{
    long item = - 1;
    wxODListCtrl *sflist = wxStaticCast(FindWindowById(ID_SHFILELIST), wxODListCtrl);
    item = sflist->GetNextItem(item, wxODLIST_NEXT_ALL, wxODLIST_STATE_SELECTED);
    if (item != ( - 1))
    {
        CKnownFile *cur_file = (CKnownFile *) sflist->GetItemData(item);
        ShowDetails(cur_file);
    }
}

void CSharedFilesWnd::ShowDetails(CKnownFile *cur_file)
{
    std::ostringstream buffer;
    pop_bartrans->SetRange(theApp.knownfiles->transfered/1024);
    pop_bartrans->SetValue(cur_file->statistic.GetTransfered() /1024);
    GetDlgItem(IDC_STRANSFERED)->SetLabel(CastItoXBytes(cur_file->statistic.GetTransfered()));
    pop_bar->SetRange(theApp.knownfiles->requested);
    pop_bar->SetValue(cur_file->statistic.GetRequests());
    buffer << cur_file->statistic.GetRequests();
    GetDlgItem(IDC_SREQUESTED)->SetLabel(wxString(buffer.str().c_str(), *wxConvCurrent));
    buffer.str("");
    buffer << cur_file->statistic.GetAccepts();
    pop_baraccept->SetRange(theApp.knownfiles->accepted);
    pop_baraccept->SetValue(cur_file->statistic.GetAccepts());
    GetDlgItem(IDC_SACCEPTED)->SetLabel(wxString(buffer.str().c_str(), *wxConvCurrent));
    GetDlgItem(IDC_STRANSFERED2)->SetLabel(CastItoXBytes(cur_file->statistic.GetAllTimeTransfered()));
    buffer.str("");
    buffer << cur_file->statistic.GetAllTimeRequests();
    GetDlgItem(IDC_SREQUESTED2)->SetLabel(wxString(buffer.str().c_str(), *wxConvCurrent));
    buffer.str("");
    buffer << cur_file->statistic.GetAllTimeAccepts();
    GetDlgItem(IDC_SACCEPTED2)->SetLabel(wxString(buffer.str().c_str(), *wxConvCurrent));
    memcpy(shownFileHash, cur_file->GetFileHash(), 16);
}

void CSharedFilesWnd::Localize()
{
}
